using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using System.Collections;

namespace OpenCL
{
    public class Regexp
    {
        public static IEnumerable matches(String pPattern, String pText, int pGroupId)
        {
            Regex r = new Regex(pPattern, RegexOptions.IgnoreCase | RegexOptions.Compiled);
            for (Match m = r.Match(pText); m.Success; m = m.NextMatch())
                yield return m.Groups[pGroupId].Value;
        }

        public static List<String> matchesAsList(String pPattern, String pText, int pGroupId)
        {
            List<String> lines = new List<String>();
            foreach (String line in matches(pPattern, pText, pGroupId))
                lines.Add(line);
            return lines;
        }

        public static String matchFirst(String pPattern, String pText, int pGroupId)
        {
            foreach (String line in matches(pPattern, pText, pGroupId))
                return line;
            return null;
        }
    }

    public class Debug
    {
        public static bool isDebug = true;
        public static StreamWriter toFile = null;

        public static void open(String fileName)
        {
//            toFile = File.CreateText(fileName);
            toFile = new StreamWriter(fileName, true);
        }

        public static void print(String msg)
        {
            if (isDebug)
            {
                if (toFile != null)
                    toFile.Write(msg);
                Console.Write(msg);
            }
        }

        public static void println(String msg)
        {
            print(msg + "\r\n");
        }

        public static void print(ICollection c)
        {
            Debug.print("[");
            foreach (Object o in c)
                Debug.print(o+" ");
            Debug.print("]");
        }

        public static void close() 
        {
            toFile.Close();
        }
    }

    public class FileSystem
    {
        public static void createDir(String filePath)
        {
            filePath = STR.replace(filePath, @"\", "/");
            String dir = STR.noLast(filePath, "/");
            Directory.CreateDirectory(dir);
        }

        public static String strToFileName(String pStr)
        {
            String s = Regex.Replace(pStr, @"[^a-zA-Z0-9]", "_");
/*            StringBuilder rzStr = new StringBuilder();
            foreach (char ch in pStr) {
                if (Char.IsPunctuation(ch) || Char.IsSymbol(ch) || Char.IsWhiteSpace(ch))
                    rzStr.Append("_");
                else
                    rzStr.Append(ch);
            }
            String s = rzStr.ToString(); */
            return s.Substring(0, Math.Min(s.Length, 64));
        }

        public static void createFile(String filePath)
        {
            FileInfo file = new FileInfo(filePath);
            if (!file.Exists) file.Create();
        }
    }

    public class TextFile
    {
        Encoding encoder = Encoding.GetEncoding("UTF-8", new EncoderExceptionFallback(), new DecoderExceptionFallback());
        public FileStream stream;
        public StreamReader reader;
        public StreamWriter writer;

        public TextFile(String filePath, FileMode mode)
        {
            stream = File.Open(filePath, mode);
            if (mode != FileMode.Append)
                reader = new StreamReader(stream);
            writer = new StreamWriter(stream, encoder);
        }

        public TextFile(String filePath) : this(filePath, FileMode.OpenOrCreate) {}

        public void write(String str) 
        {
            bool isEncodeError = false;
            try
            {
//                Encoding.UTF8.GetBytes(str);
                byte[] bytes = encoder.GetBytes(str);
//                System.Text.UTF8Encoding.getByes(
//                UTF8Encoding e = new UTF8Encoding();
//                e.GetBytes(str);
            }
            catch
            {
                isEncodeError = true;
                Console.WriteLine("Error : TextFile.writeln(" + str + ") fail !");
            }
            if (!isEncodeError)
            {
                writer.Write(str);
                flush();
            }
        }

        public void writeln(String str)
        {
            write(str + "\r\n");
        }

        public IEnumerable lines()
        {
            while (true)
            {
                String line = reader.ReadLine();
                if (line == null) break;
                yield return line;
            }
        }

        public void flush() { writer.Flush(); }
        public void close() { 
            writer.Close();  
            if (reader != null) reader.Close(); 
            stream.Close(); 
        }

        public static String fileToText(String filePath)
        {
            if (filePath.StartsWith("file:\\"))
                filePath = filePath.Substring(6);
            StreamReader file = new StreamReader(filePath);
            String text = file.ReadToEnd();
            file.Close();
            return text;
        }

        public static void textToFile(String text, String filePath)
        {
            if (filePath.StartsWith("file:\\"))
                filePath = filePath.Substring(6);
            StreamWriter file = new StreamWriter(filePath);
            file.Write(text);
            file.Close();
        }
    }

    public class STR
    {
        public static String AlphaUpper = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        public static String AlphaLower = "abcdefghijklmnopqrstuvwxyz";
        public static String Digit = "0123456789";
        public static String Symbols = "~!@#$%^&*()_+{}|:\"<>?`-=\\][';/.,";
        public static String ChineseSymbols = "CABFGHIuvyzww]^KK\\ġġEP";

        public static String replaceAnyToChar(String pStr, String pChars, char pToChar)
        {
            StringBuilder rzStr = new StringBuilder();
            for (int i = 0; i < pStr.Length; i++)
            {
                char c = pStr[i];
                if (pChars.IndexOf(c) >= 0)
                    rzStr.Append(pToChar);
                else
                    rzStr.Append(c);
            }
            return rzStr.ToString();
        }

        public static String replace(String pStr, String fromPat, String toPat)
        {
            if (fromPat.Length == 0) return pStr;
            if (pStr.IndexOf(fromPat) < 0) return pStr;
            StringBuilder rzStr = new StringBuilder();
            int strIdx = 0, nextIdx;
            while ((nextIdx = pStr.IndexOf(fromPat, strIdx)) >= 0)
            {
                rzStr.Append(pStr.Substring(strIdx, nextIdx - strIdx));
                rzStr.Append(toPat);
                strIdx = nextIdx + fromPat.Length;
            }
            rzStr.Append(pStr.Substring(strIdx));
            return rzStr.ToString();
        }

        public static String expand(String pText, String pMacros)
        {
            String[] macros = pMacros.Split('|');
            for (int i = 0; i < macros.Length; i++)
            {
                String name = head(macros[i], "=");
                String expand = tail(macros[i], "=");
                pText = replace(pText, name, expand);
            }
            return pText;
        }

        public static String innerText(String pText, String beginMark, String endMark)
        {
            int beginStart = pText.IndexOf(beginMark);
            if (beginStart < 0) return null;
            int beginEnd = beginStart + beginMark.Length;
            int endStart = pText.IndexOf(endMark, beginEnd);
            if (endStart < 0) return null;
            return pText.Substring(beginEnd, endStart - beginEnd);
        }

        public static String head(String pStr, String pSpliter)
        {
            int spliterPos = pStr.IndexOf(pSpliter);
            if (spliterPos < 0) return pStr;
            return pStr.Substring(0, spliterPos);
        }

        public static String tail(String pStr, String pSpliter)
        {
            int spliterPos = pStr.IndexOf(pSpliter);
            if (spliterPos < 0) return "";
            return pStr.Substring(spliterPos + pSpliter.Length);
        }

        public static String last(String pStr, String pSpliter)
        {
            int spliterPos = pStr.LastIndexOf(pSpliter);
            if (spliterPos < 0) return pStr;
            return pStr.Substring(spliterPos + 1);
        }

        public static String noLast(String pStr, String pSpliter)
        {
            int spliterPos = pStr.LastIndexOf(pSpliter);
            if (spliterPos < 0) return pStr;
            return pStr.Substring(0, spliterPos);
        }
        public static String maxInnerText(String pText, String beginMark, String endMark)
        {
            int beginStart = pText.IndexOf(beginMark);
            if (beginStart < 0) return null;
            int beginEnd = beginStart + beginMark.Length;
            int endStart = pText.LastIndexOf(endMark);
            if (endStart < 0) return null;
            return pText.Substring(beginEnd, endStart - beginEnd);
        }

        public static String space(int n)
        {
            return String.Format("{0," + n + "}", "");
        }

        public static int str2int(String s)
        {
            return Int32.Parse(s);
        }

        public static String int2str(int i)
        {
            return i + "";
        }

        public static String bytes2str(byte[] buf, int from, int len, String encode)
        {
            return Encoding.GetEncoding(encode).GetString(buf, from, len);
        }

        public static String bytes2str(byte[] buf, String encode)
        {
            return bytes2str(buf, 0, buf.Length, encode);
        }

        public static byte[] str2bytes(String str, String encode)
        {
            return Encoding.GetEncoding(encode).GetBytes(str);
        }

        public static String trim(String str)
        {
            return str.Trim();
        }

        public static String nonull(String str)
        {
            if (str == null) return "";
            return str;
        }

        public static String reverse(String str)
        {
            StringBuilder revStr = new StringBuilder();
            for (int i=str.Length-1; i>=0; i--)
                revStr.Append(str[i]);
            return revStr.ToString();
        }
    }
}
